###############################################################################
#
# Tolerance for root solution
#
###############################################################################
"""
    abstract type AbstractTolerance{FT}

Hierachy of AbstractTolerance:
- ResidualTolerance
- SolutionTolerance
- StepTolerance
"""
abstract type AbstractTolerance{FT} end




"""
    struct ResidualTolerance{FT}

# Fields
$(TYPEDFIELDS)
"""
struct ResidualTolerance{FT} <: AbstractTolerance{FT}
    "Tolerance for residual"
    tol::FT
    "limit of iterations"
    n_limit::Int
end




"""
    struct SolutionTolerance{FT}

# Fields
$(TYPEDFIELDS)
"""
struct SolutionTolerance{FT} <: AbstractTolerance{FT}
    "Tolerance for solution"
    tol::FT
    "limit of iterations"
    n_limit::Int
end




"""
    struct SolutionToleranceND{FT}

# Fields
$(TYPEDFIELDS)
"""
struct SolutionToleranceND{FT} <: AbstractTolerance{FT}
    "Tolerance for solution"
    tol::Array{FT,1}
    "limit of iterations"
    n_limit::Int
end
